/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//  This program selects data of a given type from a frame cache dump and 
//  prints it out as a nds2 server source list. The 

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <list>
#include <cstdlib>

//   This variable selects the use of the %r format (remove specified number 
//   of low-order digits) rather than the %d format (specified number of 
//   high-order digits of the gps) This will solve the GPS1G error.
const static bool UseRFmt = true;

typedef unsigned long gps_type;

class frame_segment {
public:
    std::string desc(void) const;
    std::string _directory;
    std::string _prefix;
    gps_type    _duration;
    gps_type    _start;
    gps_type    _stop;
    
};

typedef std::list<frame_segment> seg_list;

using namespace std;

std::string
frame_segment::desc(void) const {
    ostringstream out;
    out << _prefix << " " << _start << "-" <<_stop;
    return out.str();
}


inline std::string
slurp(std::string& s, char delim) {
    std::string::size_type inx = s.find(delim);
    std::string temp;
    if (inx == std::string::npos) {
	temp = s;
	s.clear();
    } else {
	temp = s.substr(0, inx);
	s.erase(0, inx+1);
    }
    return temp;
}

void
syntax(void) {
    cerr << "syntax: " << endl;
    cerr << "build_source_list frame-type [start-gps [end-gps]]" << endl;
}

int
main(int argc, const char* argv[]) {
    if (argc < 2) {
	syntax();
	exit(1);
    }
    string typesel = argv[1];

    gps_type gps_start(0), gps_stop(0);
    if (argc > 2) {
	gps_start = strtol(argv[2], 0, 0);
	if (argc > 3) gps_stop = strtol(argv[3], 0, 0);
    }

    seg_list myList;

    string file = "frame_cache_dump";
    ifstream inf(file.c_str());
    if (! inf.is_open() ) {
	cerr << "Unable to open file: " << file << endl;
	exit(1);
    }
    while (! inf.eof()) {
	string line;
	getline(inf, line);
	if (inf.eof()) continue;
	frame_segment seg;
	seg._directory = slurp(line, ',');
	seg._prefix    = slurp(line, ',');
	seg._prefix   += "-";
	seg._prefix   += slurp(line, ',');
	char* p = const_cast<char*>(line.c_str());
	//----------------------------------------
	// skip by number of frames for now...
	long nframe = strtol(p, &p, 0);
	if (!nframe) {
	    cerr << "Invalid number of frames in: " << seg.desc() << endl;
	}
	p++;
	//----------------------------------------
	seg._duration  = strtol(p, &p, 0);

	//cout << "Segment prefix: " << seg._prefix << endl;
	if (seg._prefix != typesel) continue;

	string::size_type inx = line.find('{');
	if (inx == string::npos) {
	    cerr << "Can't find '{'" << endl;
	    break;
	}

	p = const_cast<char*>(line.c_str() + inx + 1);
	while ( ('0' <= *p && '9' >= *p) || ' ' == *p) {
	    seg._start  = strtol(p, &p, 0);
	    seg._stop   = strtol(p, &p, 0);
	    if (gps_stop) {
		if (seg._start >= gps_stop) continue; 
		if (seg._stop  >  gps_stop) seg._stop = gps_stop;
	    }
	    if (gps_start) {
		if (seg._stop  <= gps_start) continue; 
		if (seg._start <  gps_start) seg._start = gps_start;
	    }
	    myList.push_back(seg);
	}
	if (*p != '}') {
	    cerr << "List didn't end with '}'" << endl;
	    break;
	}
    }

    //----------------------------------  Loop over entries in the list
    for (seg_list::iterator i=myList.begin(); i!=myList.end(); ++i) {
	string tmpdir = i->_directory;

	//------------------------------  Get first numeric digit at end of
	//                                the directory name
	string::size_type inum = tmpdir.find_last_not_of("0123456789");
	if (inum == string::npos) continue;
	inum++;
	string::size_type lnum = tmpdir.size() - inum;
	if (!lnum) continue;

	//------------------------------  Compare to first digits of start gps
	ostringstream start_stream;
	start_stream << i->_start;
	std::string start_str = start_stream.str();
	
	if (tmpdir.substr(inum, lnum) != start_str.substr(0,lnum)) continue;

	//------------------------------
	ostringstream dir_str;
	if (UseRFmt) {
	    dir_str << tmpdir.substr(0,inum) << "%" << start_str.size() - lnum 
		    << "r";
	} else {
	    dir_str << tmpdir.substr(0,inum) << "%" << lnum << "d";
	}
	i->_directory = dir_str.str();
    }

    seg_list::iterator i=myList.begin();
    seg_list::iterator j=i++;
    while ( i != myList.end() ) {
	if ( i->_start     == j->_stop && 
	     i->_directory == j->_directory && 
	     i->_duration  == j->_duration) {
	    j->_stop = i->_stop;
	    i = myList.erase(i);
	} else {
#if 0
	    if (i->_start != j->_stop) {
		cout << "Gap " << j->_stop << "-" <<  i->_start 
		     << " ("   << i->_start - j->_stop << "s) in "
		     << i->_directory << endl;
	    }
#endif
	    j = i++;
	}
    }
    for (seg_list::const_iterator i=myList.begin(); i!=myList.end(); ++i) {
	string archtype;
	if (i->_directory.substr(0,9) == "/archive/") {
	    archtype = "archive";
	} else if (i->_directory.substr(0,6) == "/data/") {
	    archtype = "cache";
	} else {
	    continue;
	}
	cout << i->_prefix << " " << i->_start << " " << i->_stop
	     << " " << i->_duration << " " << archtype << " - " 
	     << i->_directory << endl;
    }
}
